@@ -1,10 +1,12 @@
 //  MaCoPiX = Mascot Construnctive Pilot for X
 //                                (ActX / Gtk+ Evolution)
-//    A unified desktop mascot program
-//    for UNIX / X Window System with Gdk Environment
 //
-//       balloon.c  : Drawing Balloon Message of mascots for   MaCoPiX
-//                                         copyright K.Chimari 2002-4
+//
+//     balloon.c
+//     Creating and Drawing Balloon Message Windows
+//
+//                            Copyright 2002-2007  K.Chimari
+//                                     http://rosegray.sakura.ne.jp/
 //
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,16 +21,19 @@
 // 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 //
 
+
 #include "main.h"
 #include<string.h>
 #include<ctype.h>
 
-
 // *** GLOBAL ARGUMENT ***
 extern GtkWidget *win_main, *balloon_main;
+#ifdef USE_WIN32
+extern GtkWidget *balloon_fg;
+#endif
 extern GtkWidget *popup_menu; 
 extern gint window_x, window_y;
 extern GdkDrawable *pixmap_bal[2];
@@ -38,14 +43,20 @@ extern void MoveBalloon();
 
 extern gboolean flag_balloon;
 
-GdkBitmap *mask_bal[2]={NULL,NULL};
-static gint old_x=0, old_y=0;
+extern GdkBitmap *mask_bal[2];
+
+gint old_x, old_y;
 
 // *** Protptype of function in this file
 void DrawBalloon();
 void DoBalloon();
 GtkWidget * make_balloon();
+#ifdef USE_WIN32
+GtkWidget * make_balloon_fg();
+#endif
+void balloon_arg_init();
 
+void DoSysBalloon();
 
 void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
 {
@@ -62,10 +73,10 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   PangoLayout *pango_text;
   PangoLanguage *pango_lang;
 #endif
-
+  
   work_page=mascot->bal_page;
   work_page^=1;
-
+  
   // bal_height, bal_width  ƥȰԤ礭
   bal_height=0;
   bal_width=0;
@@ -99,36 +110,37 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
     +mascot->baltext_y+mascot->wbalbd*2;
   mascot->balwidth=w;
   mascot->balheight=h+h_arrow;
-
+  
   // Х롼ΰַ
   mascot->bal_pos=mascot->bal_defpos;
   if(mascot->bal_pos==BAL_POS_LEFT){
-    if((mascot->x-w)<0) mascot->bal_pos=BAL_POS_RIGHT;
+    if((mascot->x-w+mascot->bal_lxoff[mascot->anime_ptn])<0) 
+      mascot->bal_pos=BAL_POS_RIGHT;
   }
   else if(mascot->bal_pos==BAL_POS_RIGHT){
-    if((mascot->x+mascot->width+w)>mascot->width_root)
+    if((mascot->x+mascot->width+w+mascot->bal_lxoff[mascot->anime_ptn])
+       >mascot->width_root)
       mascot->bal_pos=BAL_POS_LEFT;
   }
 
-  //gtk_widget_unmap(balloon_main);
-  //MoveBalloon(mascot,mascot->x,mascot->y);
     
-    
-  // Х롼Υꥵ
-  gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_baldraw), w,h+h_arrow);
-  gdk_window_resize (balloon_main->window,w,h+h_arrow);
-
-
-
-  
   if (pixmap_bal[work_page]) {
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(pixmap_bal[work_page]));
+#else
     gdk_pixmap_unref(pixmap_bal[work_page]);
+#endif
   } 
-  
+
+ 
   pixmap_bal[work_page] = gdk_pixmap_new(balloon_main->window,w,h+h_arrow,-1);
   
   if (mask_bal[work_page]) {
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(mask_bal[work_page]));
+#else
     gdk_pixmap_unref(mask_bal[work_page]);
+#endif
   } 
   
   mask_bal[work_page] = gdk_pixmap_new(balloon_main->window,
@@ -148,10 +160,32 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   gdk_gc_get_values(mascot->gc_balmask[work_page], &gcv);
   gdk_gc_set_foreground(mascot->gc_balmask[work_page], &gcv.background);
   gdk_gc_set_background(mascot->gc_balmask[work_page], &gcv.foreground);
-    
+  
     
   // **** pixmapؤBUFFERING
   // border
+#ifdef USE_WIN32
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+	       TRUE,
+	       0,0,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+	       0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-1,0,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+	       0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+	       TRUE,
+	       0,(h+2)-mascot->baltext_y*2-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+	       0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+	       0,5760*4);
+#else
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
 	       TRUE,
 	       0,0,
@@ -169,9 +203,10 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
 	       5760*2,5760);
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
 	       TRUE,
-	       w-mascot->baltext_x*2-1,h-mascot->baltext_y*2-1,
+	       w-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
 	       5760*3,5760);
+#endif
   gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbd[work_page],
 		     TRUE,
 		     0,mascot->baltext_y,
@@ -202,26 +237,49 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
 		   TRUE,points,4);
   
   // Mask (BorderƱ)
+#ifdef USE_WIN32
   gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
 	       TRUE,
 	       0,0,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760,5760);
+               0,5760*4);
+  gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-1,0,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+  	       0,5760*4);
+  gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
+	       TRUE,
+	       0,(h+2)-mascot->baltext_y*2-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+  	       0,5760*4);
+  gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+               0,5760*4);
+#else
+  gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
+	       TRUE,
+	       0,0,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+               5760,5760);
   gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
 	       TRUE,
 	       w-mascot->baltext_x*2-1,0,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760);
+  	       0,5760);
   gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
 	       TRUE,
 	       0,h-mascot->baltext_y*2-1,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*2,5760);
+  	       5760*2,5760);
   gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
 	       TRUE,
 	       w-mascot->baltext_x*2-1,h-mascot->baltext_y*2-1,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*3,5760);
+               5760*3,5760);
+#endif
   gdk_draw_rectangle(mask_bal[work_page],mascot->gc_balmask[work_page],
 		     TRUE,
 		     0,mascot->baltext_y,
@@ -234,29 +292,55 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
 		   TRUE,points,4);
     
   // background
+#ifdef USE_WIN32
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 	       TRUE,
 	       mascot->wbalbd,mascot->wbalbd,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760,5760);
+               0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-mascot->wbalbd-1,
+	       mascot->wbalbd,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+  	       0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
+	       TRUE,
+	       mascot->wbalbd,
+	       (h+2)-mascot->baltext_y*2-mascot->wbalbd-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+  	       0,5760*4);
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
+	       TRUE,
+	       (w+2)-mascot->baltext_x*2-mascot->wbalbd-1,
+	       (h+2)-mascot->baltext_y*2-mascot->wbalbd-1,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+  	       0,5760*4);
+#else
+  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
+	       TRUE,
+	       mascot->wbalbd,mascot->wbalbd,
+	       mascot->baltext_x*2,mascot->baltext_y*2,
+               5760,5760);
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 	       TRUE,
 	       w-mascot->baltext_x*2-mascot->wbalbd-1,
-	       mascot->wbalbd,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760);
+ 	       mascot->wbalbd,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760);
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 	       TRUE,
 	       mascot->wbalbd,
 	       h-mascot->baltext_y*2-mascot->wbalbd-1,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*2,5760);
+  	       5760*2,5760);
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 	       TRUE,
 	       w-mascot->baltext_x*2-mascot->wbalbd-1,
 	       h-mascot->baltext_y*2-mascot->wbalbd-1,
 	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*3,5760);
+  	       5760*3,5760);
+#endif
   gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 		     TRUE,
 		     mascot->wbalbd,
@@ -308,10 +392,73 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   }
   mascot->bal_page=work_page;
 
-  MoveBalloon(mascot,mascot->x,mascot->y);
-  gtk_widget_map(balloon_main);
-  while (g_main_iteration(FALSE));
+  //  Balloon BG Draw  
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gdk_window_set_back_pixmap(mascot->w_baldraw_fg->window,
+			       pixmap_bal[mascot->bal_page],
+			       FALSE);
+  }
+#endif
 
+  gdk_window_set_back_pixmap(mascot->w_baldraw->window,
+			     pixmap_bal[mascot->bal_page],
+			     FALSE);
+
+  gdk_flush();
+
+
+ 
+  // Resize of Balloon (calling dw_bal_expose)
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_baldraw_fg), 
+			   w,
+			   h+h_arrow);
+  }
+#endif
+  gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_baldraw), 
+			 w,
+			 h+h_arrow);
+
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gdk_window_resize (balloon_fg->window,w,h+h_arrow);
+  }
+#endif
+  gdk_window_resize (balloon_main->window,w,h+h_arrow);
+
+
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gtk_widget_shape_combine_mask( balloon_fg,
+				   mask_bal[mascot->bal_page],
+				   0,0);
+  }
+#endif
+
+  gtk_widget_shape_combine_mask( balloon_main,
+				 mask_bal[mascot->bal_page],
+				 0,0);
+
+
+  //  End of Balloon Draw  
+
+  gdk_flush();
+
+  MoveBalloon(mascot,mascot->x,mascot->y);
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gtk_widget_map(balloon_fg);
+  }
+#endif
+  gtk_widget_map(balloon_main);
+  // ToolTipsѻ˺ for v1.4.0
+  //while (g_main_iteration(FALSE));
+
+#ifdef USE_GTK2
+  g_object_unref(G_OBJECT(pango_text));
+#endif
   gdk_flush();
 
 }
@@ -327,6 +474,7 @@ void DoBalloon(typMascot *mascot)
   static gint wp_no, wn_no;
   static int balseqp;
   static int balseqp_biff;
+  static int balseqp_sys;
   static int balseqp_sock;
   static int balseqpend[64];
   int i_p;
@@ -353,6 +501,32 @@ void DoBalloon(typMascot *mascot)
 		w_tmp_m, mascot->mail.count, w_tmp_m2);
 	strcpy(w_tmp, w_tmp_m3);
       }
+      while(NULL != (w_tmp_m2 = strstr(w_tmp, "%f"))){  // Last From
+	mp = strlen(w_tmp) - strlen(w_tmp_m2); 
+	strncpy(w_tmp_m, w_tmp, mp);
+	w_tmp_m[mp] = '\0';
+	w_tmp_m2 +=  strlen("%f");  //Last Sbject
+	if(mascot->mail.last_f!=NULL)
+	   sprintf(w_tmp_m3,"%s%s%s",
+		   w_tmp_m, mascot->mail.last_f, w_tmp_m2);
+	else
+	  sprintf(w_tmp_m3,_("%s(From is not avairable)%s"),
+		   w_tmp_m, w_tmp_m2);
+	strcpy(w_tmp, w_tmp_m3);
+      }
+      while(NULL != (w_tmp_m2 = strstr(w_tmp, "%s"))){
+	mp = strlen(w_tmp) - strlen(w_tmp_m2); 
+	strncpy(w_tmp_m, w_tmp, mp);
+	w_tmp_m[mp] = '\0';
+	w_tmp_m2 +=  strlen("%s");
+	if(mascot->mail.last_s!=NULL)
+	  sprintf(w_tmp_m3,"%s%s%s",
+		  w_tmp_m, mascot->mail.last_s, w_tmp_m2);
+	else
+	  sprintf(w_tmp_m3,_("%s(Subject is not avairable)%s"),
+		   w_tmp_m, w_tmp_m2);
+	strcpy(w_tmp, w_tmp_m3);
+      }
       balseqp_biff=BALLOON_EXPIRE;
       break;
     case BALLOON_POPERROR:
@@ -361,15 +535,19 @@ void DoBalloon(typMascot *mascot)
       break;
     case BALLOON_SOCKMSG:
       strncpy(w_tmp,mascot->sockmsg,1023);
-      balseqp_sock=SOCK_BALLOON_EXPIRE;
+      balseqp_sock=mascot->sockmsg_expire;
       break;     
     case BALLOON_DUET:
       strncpy(w_tmp,mascot->sockmsg,1023);
       break;
+    case BALLOON_SYS:
+      strncpy(w_tmp,mascot->sysmsg,1023);
+      balseqp_sys=SYS_BALLOON_EXPIRE;
+      break;
     default:
       strcpy(w_tmp,mascot->click_word[mascot->anime_ptn]);
     }
-
+ 
     wp[wp_no]=(char *)strtok(w_tmp,"%");
 
     for(;;){
@@ -491,12 +669,16 @@ void DoBalloon(typMascot *mascot)
     else{ // Х롼
       if(mascot->bal_mode==BALLOON_SOCKMSG) {
         balseqp=-3;
-      } else if((mascot->bal_mode==BALLOON_MAIL)
-	 ||(mascot->bal_mode==BALLOON_POPERROR)){
+      }
+      else if(mascot->bal_mode==BALLOON_SYS) {
+        balseqp=-4;
+      }
+      else if((mascot->bal_mode==BALLOON_MAIL)
+	      ||(mascot->bal_mode==BALLOON_POPERROR)){
 	balseqp=-2;
       }
       else{
-	balseqp=-1;
+         balseqp=-1;
       }
     }
   }
@@ -512,14 +694,21 @@ void DoBalloon(typMascot *mascot)
     }
     else if(balseqp==-2){
       if(balseqp_biff==0){  // BiffѥХ롼Υ
+#ifdef USE_WIN32
+	gtk_widget_unmap(balloon_fg);
+#endif
 	gtk_widget_unmap(balloon_main);
 	flag_balloon=FALSE;
       }
       else{
 	balseqp_biff--;
       }
-    } else if(balseqp==-3){
+    } 
+    else if(balseqp==-3){
       if(balseqp_sock==0){  // SocketѥХ롼Υ
+#ifdef USE_WIN32
+	gtk_widget_unmap(balloon_fg);
+#endif
 	gtk_widget_unmap(balloon_main);
 	flag_balloon=FALSE;
       }
@@ -527,6 +716,18 @@ void DoBalloon(typMascot *mascot)
 	balseqp_sock--;
       }
     }
+    else if(balseqp==-4){
+      if(balseqp_sys==0){  // SysѥХ롼Υ
+#ifdef USE_WIN32
+	gtk_widget_unmap(balloon_fg);
+#endif
+	gtk_widget_unmap(balloon_main);
+	flag_balloon=FALSE;
+      }
+      else{
+	balseqp_sys--;
+      }
+    } 
   }
   old_x=mascot->x;
   old_y=mascot->y;
@@ -534,12 +735,19 @@ void DoBalloon(typMascot *mascot)
 }
 
 
+
+
 GtkWidget * make_balloon(void){
   GtkWidget *balloon_vbox;
   GtkWidget *balloon_drawing_area;
 
   balloon_main = gtk_window_new(GTK_WINDOW_POPUP);
+#ifdef USE_GTK2
+  gtk_window_set_accept_focus(GTK_WINDOW(balloon_main),FALSE);
+#endif
   gtk_widget_set_app_paintable(balloon_main, TRUE);
+  gtk_window_set_wmclass(GTK_WINDOW(balloon_main), "balloon", "MaCoPiX");
+
   gtk_widget_set_events(GTK_WIDGET (balloon_main), 
 			GDK_FOCUS_CHANGE_MASK | 
 			GDK_BUTTON_MOTION_MASK | 
@@ -548,7 +756,10 @@ GtkWidget * make_balloon(void){
 			GDK_EXPOSURE_MASK);
   gtk_widget_realize(balloon_main);
   gdk_window_set_decorations(balloon_main->window, 0);
+#ifndef USE_WIN32
+  /* gdk_window_set_override_redirect is not implemented (for warning) */
   gdk_window_set_override_redirect(balloon_main->window,TRUE);
+#endif
 
   balloon_vbox = gtk_hbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER (balloon_main), balloon_vbox);
@@ -564,3 +775,61 @@ GtkWidget * make_balloon(void){
 
   return(balloon_drawing_area);
 }
+
+#ifdef USE_WIN32
+GtkWidget * make_balloon_fg(void){
+  GtkWidget *balloon_vbox;
+  GtkWidget *balloon_drawing_area;
+
+  balloon_fg = gtk_window_new(GTK_WINDOW_POPUP);
+#ifdef USE_GTK2
+  gtk_window_set_accept_focus(GTK_WINDOW(balloon_fg),FALSE);
+#endif
+  gtk_widget_set_app_paintable(balloon_fg, TRUE);
+  gtk_window_set_wmclass(GTK_WINDOW(balloon_fg), "balloon", "MaCoPiX");
+
+  gtk_widget_set_events(GTK_WIDGET (balloon_fg), 
+			GDK_FOCUS_CHANGE_MASK | 
+			GDK_BUTTON_MOTION_MASK | 
+			GDK_BUTTON_RELEASE_MASK | 
+			GDK_BUTTON_PRESS_MASK | 
+			GDK_EXPOSURE_MASK);
+  gtk_widget_realize(balloon_fg);
+  gdk_window_set_decorations(balloon_fg->window, 0);
+
+  balloon_vbox = gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER (balloon_fg), balloon_vbox);
+  gtk_widget_show(balloon_vbox);
+
+  balloon_drawing_area = gtk_drawing_area_new();
+  gtk_drawing_area_size (GTK_DRAWING_AREA(balloon_drawing_area), 1, 1);
+  gtk_box_pack_start(GTK_BOX(balloon_vbox), balloon_drawing_area,
+		     TRUE, TRUE, 0);
+  gtk_widget_show(balloon_drawing_area);
+
+
+
+  return(balloon_drawing_area);
+}
+#endif
+
+
+void balloon_arg_init(){
+  old_x=0;
+  old_y=0;
+}
+
+
+void DoSysBalloon(typMascot *mascot, gchar *buf)
+{
+
+  mascot->balseq=0;
+  mascot->bal_mode=BALLOON_SYS;
+  
+  if (mascot->sysmsg != NULL)
+    g_free(mascot->sysmsg);
+  mascot->sysmsg = g_strdup(buf);
+  DoBalloon(mascot);
+  flag_balloon=TRUE;
+}
+
