@@ -1,10 +1,12 @@
 //  MaCoPiX = Mascot Construnctive Pilot for X
 //                                (ActX / Gtk+ Evolution)
-//    A unified desktop mascot program
-//    for UNIX / X Window System with Gdk Environment
 //
-//       callbacks.c  : Callback Functions for   MaCoPiX
-//                                         copyright K.Chimari 2002-4
+//
+//     callback.c
+//     Callback functions
+//
+//                            Copyright 2002-2007  K.Chimari
+//                                     http://rosegray.sakura.ne.jp/
 //
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,20 +21,37 @@
 // 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 //
 
+
 #include "main.h"
 #include <signal.h>
+
+#if HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
+
+#ifdef USE_WIN32
+#include <windows.h>
+#include <gdk/gdkwin32.h>
+#include <mmsystem.h>
+#else // for WIN32
 #include <gdk/gdkx.h>
+#endif
 
 #include "sockmsg.h"
 
 #undef DEBUG
 
 // *** GLOBAL ARGUMENT ***
-extern GtkWidget *win_main,*balloon_main,*clock_main,*biff_pix;
+extern GtkWidget *win_main,*balloon_main,*clock_main;
+#ifdef USE_WIN32
+extern GtkWidget *clock_fg, *balloon_fg;
+#endif
+#ifdef USE_BIFF
+extern GtkWidget *biff_pix;
+#endif
 extern GtkTooltips *tooltip;
 extern GtkWidget *PopupMenu; 
 extern GdkDrawable *pixmap_main[2], *pixmap_clk[2], *pixmap_bal[2];
@@ -48,54 +67,80 @@ extern void DoBalloon();
 
 extern void AllRandomChangeMascotMenu();
 
+#ifdef USE_BIFF
 extern void create_biff_dialog();
+extern void display_biff_balloon();
+#endif
 
 extern void copy_file();
+extern gboolean my_main_iteration();
 
 
 // *** GLOBAL ARGUMENT in this code *** 
-int pre_hour=-1, pre_min=-1, pre_sec=-1;
-gint start_x=0, start_y=0, biff_start_x=0, biff_start_y=0, clk_start_x=0, clk_start_y=0;
-gboolean flag_balloon=FALSE;
-gboolean flag_anime=FALSE, flag_clock=FALSE;
-int old_day=0;
-int errflag=-1;
+int pre_hour, pre_min, pre_sec;
+gint start_x, start_y, clk_start_x, clk_start_y;
+#ifdef USE_BIFF
+gint biff_start_x, biff_start_y;
+#endif
+gboolean flag_balloon;
+gboolean flag_anime, flag_clock;
+int old_day;
+int errflag;
 int xpop,ypop;
 gint FWinX,FWinWidth;
-gint window_x, window_y, biff_window_x, biff_window_y, clk_window_x, clk_window_y;
+gint window_x, window_y, clk_window_x, clk_window_y;
+#ifdef USE_BIFF
+gint biff_window_x, biff_window_y;
+#endif
+#ifdef USE_WIN32
+HWND hWnd_active;
+HWND hWndTaskBar;
+#endif
 
 
 // Prototype of functions in this file
 int weight_random();
 int weight_click();
 
+#ifndef USE_WIN32
 int ehandler ();
+#endif
 
 void MoveMascot();
+#ifdef USE_BIFF
 void MoveBiffPix();
+#endif
 void MoveBalloon();
 int MoveToFocus();
 
-GdkGC *CreatePen();
+GdkGC *MPCreatePen();
 
 void drag_begin();
 void drag_end();
 void clk_drag_begin();
 void clk_drag_end();
+#ifdef USE_BIFF
 void biff_drag_begin();
 void biff_drag_end();
+#endif
 
 gint dw_configure_main();
+#ifdef USE_BIFF
 gint dw_configure_biff_pix();
+#endif
 gint dw_configure_clk();
 gint dw_configure_bal();
 gint dw_expose_main();
+#ifdef USE_BIFF
 gint dw_expose_biff_pix();
+#endif
 gint dw_expose_clk();
 gint dw_expose_bal();
 void window_motion();
 void clk_window_motion();
+#ifdef USE_BIFF
 void biff_window_motion();
+#endif
 void focus_in();
 void focus_out();
 gint time_update();
@@ -105,7 +150,20 @@ gchar * ini_week();
 
 void ext_play();
 void sound_play();
+#ifndef USE_WIN32
 void ChildTerm();
+#endif // USE_WIN32
+
+void raise_all();
+
+void callbacks_arg_init();
+
+#ifdef USE_WIN32
+int GetTaskbarHeight( );
+int GetTaskbarWidth( );
+gboolean IsWin32Root();
+#endif
+
 
 // ŤߤĤѥ
 int weight_random(typMascot *mascot)
@@ -139,12 +197,13 @@ int weight_click(typMascot *mascot)
 
 
 // XEvent ϢΥ顼ϥɥ顼  MoveToFocusǻȤ
+#ifndef USE_WIN32
 int ehandler (Display *d, XErrorEvent *e)
 {
     errflag = 1;
     return(0);
 }
-
+#endif
 
 // ޥåȤΰư
 void MoveMascot(typMascot *mascot, gint x, gint y)
@@ -157,21 +216,26 @@ void MoveMascot(typMascot *mascot, gint x, gint y)
 
   MoveBalloon(mascot, x, y);
   if(mascot->clkmode==CLOCK_PANEL){
+#ifdef USE_WIN32
+    gdk_window_move(clock_fg->window, 
+		    x+mascot->clk_x*((gfloat)(mascot->magnify)/100), 
+		    y+mascot->clk_y*((gfloat)(mascot->magnify)/100));
+#endif
     gdk_window_move(clock_main->window, 
 		    x+mascot->clk_x*((gfloat)(mascot->magnify)/100), 
 		    y+mascot->clk_y*((gfloat)(mascot->magnify)/100));
   }	 
+#ifdef USE_BIFF
   MoveBiffPix(mascot, x, y);
+#endif
   
-  gdk_window_raise(win_main->window);
-  gdk_window_raise(clock_main->window);
-  gdk_window_raise(balloon_main->window);
-  gdk_window_raise(biff_pix->window);
+  raise_all();
 
   gdk_flush();
 
 }
 
+#ifdef USE_BIFF
 // Biffΰư
 void MoveBiffPix(typMascot *mascot, gint x, gint y)
 {
@@ -193,22 +257,35 @@ void MoveBiffPix(typMascot *mascot, gint x, gint y)
 
  gdk_flush();
 }
-
+#endif
 
 // Х롼ΰư
 void MoveBalloon(typMascot *mascot, gint x, gint y)
 {
   gint bal_y;
-  bal_y=y;
+  bal_y=y+mascot->bal_lyoff[mascot->anime_ptn];
 
   if(bal_y<0) bal_y=0;
 
   if(mascot->bal_pos==BAL_POS_LEFT){
+#ifdef USE_WIN32
+    gdk_window_move(balloon_fg->window, 
+		    x-mascot->balwidth+mascot->bal_lxoff[mascot->anime_ptn],
+		    bal_y);
+#endif
     gdk_window_move(balloon_main->window, 
-		    x-mascot->balwidth, bal_y);
+		    x-mascot->balwidth+mascot->bal_lxoff[mascot->anime_ptn],
+		    bal_y);
   }
   else{
-    gdk_window_move(balloon_main->window, x+mascot->width, bal_y);
+#ifdef USE_WIN32
+    gdk_window_move(balloon_fg->window, 
+		    x+mascot->width+mascot->bal_rxoff[mascot->anime_ptn],
+		    bal_y);
+#endif
+    gdk_window_move(balloon_main->window, 
+		    x+mascot->width+mascot->bal_rxoff[mascot->anime_ptn],
+		    bal_y);
   }
 
  gdk_flush();
@@ -217,16 +294,47 @@ void MoveBalloon(typMascot *mascot, gint x, gint y)
 // եWindowɽư
 int MoveToFocus(typMascot *mascot, gboolean force_fl)
 {
+#ifdef USE_WIN32
+  /* for Windows Ȥꤢѥ̤ */
+  HWND   hWnd;
+  RECT   nRect;
+  DWORD dwStyle;
+
+  int sx,sy,x_root=0,y_root=0;
+  int realXPos;
+  unsigned int width;
+#else
   Window rootwin,parent,*children,child,wf;
   int sx,sy,x,y,i,x_root=0,y_root=0;
   unsigned int border,depth;
   unsigned int nchildren;
   unsigned int width,height,width_root,height_root;
   int rx,ry,wx,wy,mask_return,realXPos;
-  gboolean eflag=FALSE, flag_homepos=FALSE;
+#endif
+  gboolean eflag=FALSE;
+  gint flag_homepos=HOMEPOS_NEVER;
 
   gint oxpop,oypop;
 
+#ifdef USE_WIN32
+  /* for Windows */
+  hWnd = GetForegroundWindow();
+
+  if(hWnd!=hWnd_active){ // for multiple running 
+    char wtitle[256];    
+    GetWindowText(hWnd, wtitle, 256);
+    if(strcmp(wtitle,"MaCoPiX")==0){
+      hWnd=hWnd_active;
+    }
+    else if(mascot->no_capbar){  // for windows w/o title bar
+      DWORD wStyle;
+      wStyle = GetWindowLong (hWnd,GWL_STYLE);
+      if(!(wStyle&WS_CAPTION)){
+	hWnd=hWnd_active;
+      }
+    }
+  }
+#else
   XGetInputFocus(GDK_WINDOW_XDISPLAY(win_main->window),
 		 &wf, &i);
   XGetGeometry(GDK_WINDOW_XDISPLAY(win_main->window),
@@ -240,13 +348,46 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	       &depth);
 
   parent = 0;
+#endif
+
   sx = 0;
   sy = 0;
 
+#ifdef USE_WIN32
+    /* for Windows */
+  if(hWnd!=GDK_WINDOW_HWND(win_main->window) 
+     && hWnd!=GDK_WINDOW_HWND(balloon_fg->window)
+     && hWnd!=GDK_WINDOW_HWND(balloon_main->window)
+     && hWnd!=GDK_WINDOW_HWND(clock_fg->window)
+     && hWnd!=GDK_WINDOW_HWND(clock_main->window)
+#ifdef USE_BIFF
+     && hWnd!=GDK_WINDOW_HWND(biff_pix->window)
+#endif
+     && (int)hWnd!=0){
+
+    if(mascot->task_force){  // force to sit on the task bar
+      if((hWnd!=hWnd_active)&&(hWnd!=hWndTaskBar)){
+	  raise_all();
+      }
+      hWnd_active=hWnd;
+      hWnd=hWndTaskBar;
+    }
+    else{
+      hWnd_active=hWnd;
+    }
+      
+    GetWindowRect(hWnd, &nRect);
+    sx = nRect.left;
+    sy = nRect.top;
+    width = nRect.right-nRect.left;
+
+#else
   if(wf!=GDK_WINDOW_XWINDOW(win_main->window) 
      && wf!=GDK_WINDOW_XWINDOW(balloon_main->window)
      && wf!=GDK_WINDOW_XWINDOW(clock_main->window)
+#ifdef USE_BIFF
      && wf!=GDK_WINDOW_XWINDOW(biff_pix->window)
+#endif
      && (int)wf!=0){
 
     if(mascot->focus_autobar==TRUE){ // ȥСΥư 
@@ -356,6 +497,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	errflag = 0;
 	return(0);
       }
+
       if(errflag ==0){
 	XQueryPointer(GDK_WINDOW_XDISPLAY(win_main->window),
 		      wf, &rootwin, &child, &rx, &ry,
@@ -369,6 +511,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       sx=rx-wx;
       sy=ry-wy-mascot->bar_size;
     }
+#endif
 
     // sx, sy ϸߤΥեWindowκ(ޥȥС)а
       
@@ -406,31 +549,33 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
     }
 
     if(mascot->focus_autobar){
+#ifdef USE_WIN32   // եʤ 
+      if(IsWin32Root(hWnd, mascot->width_root, mascot->height_root)){
+#else
       if((sx==ROOTOFF_X-width)
 	 &&(sy==ROOTOFF_Y-mascot->bar_offset)){  // եʤ 
-	if(mascot->homepos!=HOMEPOS_NEVER){
-	  flag_homepos=TRUE;
-	}
+#endif
+	flag_homepos=mascot->homepos_nf;
       }
       else if(ypop<ROOTOFF_Y){ //̾˥ޥåȤϤߤǤ
-	if(mascot->homepos==HOMEPOS_OUT){
-	  flag_homepos=TRUE;
-	}
+	flag_homepos=mascot->homepos_out;
       }
     }
     else{
+#ifdef USE_WIN32   // եʤ 
+      if(IsWin32Root(hWnd, mascot->width_root, mascot->height_root)){
+#else
       if((sx==ROOTOFF_X) &&(sy==ROOTOFF_Y)){  // եʤ 
-	if(mascot->homepos!=HOMEPOS_NEVER){
-	  flag_homepos=TRUE;
-	}
+#endif
+	flag_homepos=mascot->homepos_nf;
       }
       else if(ypop<ROOTOFF_Y){ //̾˥ޥåȤϤߤǤ
-	if(mascot->homepos==HOMEPOS_OUT){
-	  flag_homepos=TRUE;
-	}
+	flag_homepos=mascot->homepos_out;
       }
     }
-    if(flag_homepos){
+   
+    switch(flag_homepos){
+    case HOMEPOS_BAR:
       if(mascot->ff_side==FF_SIDE_RIGHT){
 	if(mascot->flag_xp){
 	  xpop=((mascot->width_root-mascot->home_x)-mascot->width)
@@ -450,10 +595,21 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	  xpop=mascot->home_x-mascot->xoff+mascot->offset;
 	}
       }
+#ifdef USE_WIN32
+      mascot->home_y=GetTaskbarHeight();
+      mascot->home_x=mascot->width_root-GetTaskbarWidth();
+#endif      
       ypop=(mascot->height_root-mascot->home_y)
 	+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height;
-    }
+      break;
+      
+    case HOMEPOS_VANISH:
+      ypop=mascot->height_root;
+      break;
 
+    default:
+      break;
+    }
     
     if((oxpop!=xpop)||(oypop!=ypop)){
       MoveMascot(mascot, xpop, ypop);
@@ -478,8 +634,52 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       }
     }
   }
-
-  while (g_main_iteration(FALSE));
+      /*
+#ifdef USE_WIN32
+  else if(mascot->task_force){
+    // Stationed on Task Bar for Windows
+    if(mascot->task_force){
+      oxpop=xpop;
+      oypop=ypop;
+      
+      
+      if(mascot->ff_side==FF_SIDE_RIGHT){
+	if(mascot->flag_xp){
+	  xpop=((mascot->width_root-mascot->home_x)-mascot->width)
+	    *(1-(mascot->offsetp*0.01)) -mascot->xoff;
+	}
+	else{
+	  xpop=(mascot->width_root-mascot->home_x)
+	    -mascot->width-mascot->xoff-mascot->offset;
+	}
+      }
+      else{
+	if(mascot->flag_xp){
+	  xpop=((mascot->width_root-mascot->home_x)-mascot->width)
+	    *(mascot->offsetp*0.01) -mascot->xoff;
+	}
+	else{
+	  xpop=mascot->home_x-mascot->xoff+mascot->offset;
+	}
+      }
+      
+      mascot->home_y=GetTaskbarHeight();
+      mascot->home_x=mascot->width_root-GetTaskbarWidth();
+      
+      ypop=(mascot->height_root-mascot->home_y)
+	+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height;
+      
+      if((oxpop!=xpop)||(oypop!=ypop)){
+	MoveMascot(mascot, xpop, ypop);
+	
+	FWinWidth=mascot->width_root-mascot->home_x;
+	FWinX=0;
+      }
+    }
+  }
+#endif
+      */
+  while (my_main_iteration(FALSE));
   gdk_flush();
   return(1);
   
@@ -487,7 +687,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 
 
 // ĤGCκȥ顼allocation
-GdkGC *CreatePen(GdkDrawable *pixmap, GdkColor *color)
+GdkGC *MPCreatePen(GdkDrawable *pixmap, GdkColor *color)
 {
   GdkGC *gc;
 
@@ -540,10 +740,7 @@ void drag_end(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
 
   mascot=(typMascot *)gdata; 
 
-  gdk_window_raise(win_main->window);
-  gdk_window_raise(clock_main->window);
-  gdk_window_raise(balloon_main->window);
-  gdk_window_raise(biff_pix->window);
+  raise_all();
   
   if (mascot->drag){
     mascot->drag = FALSE;
@@ -572,11 +769,13 @@ void drag_end(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
 	flag_balloon=TRUE;
       }
 
+#ifdef USE_SOCKMSG
       // Duet˥
       if(mascot->duet_use_click){
 	mascot->duet_seq=mascot->duet_delay[mascot->anime_ptn];
 	mascot->duet_mode=DUET_CLICK;
       }
+#endif
 
       // flag_anime=TRUE;
     }
@@ -604,7 +803,7 @@ void clk_drag_begin(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
     clk_window_x = event->x;
     clk_window_y = event->y;
     gdk_window_get_pointer(NULL, &clk_start_x, &clk_start_y, &modmask);
-    gdk_pointer_grab(clock_main->window, 
+    gdk_pointer_grab(widget->window, 
 		     FALSE, GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 		     NULL, NULL, GDK_CURRENT_TIME);
     gdk_flush();
@@ -623,22 +822,20 @@ void clk_drag_end(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
 
   mascot=(typMascot *)gdata; 
 
-  gdk_window_raise(win_main->window);
-  gdk_window_raise(clock_main->window);
-  gdk_window_raise(balloon_main->window);
-  gdk_window_raise(biff_pix->window);
+  raise_all();
   
   if (mascot->clk_drag){
     mascot->clk_drag = FALSE;
     gdk_window_get_pointer(NULL, &end_x, &end_y, &modmask);
   }
   
-  gdk_window_set_cursor(clock_main->window,mascot->cursor.clk);
+  gdk_window_set_cursor(widget->window,mascot->cursor.clk);
   gdk_pointer_ungrab(GDK_CURRENT_TIME);
   gdk_flush();
 }
 
 
+#ifdef USE_BIFF
 // Biff ޥܥ򤪤ȤΥ٥Callback
 void biff_drag_begin(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
 {
@@ -673,10 +870,7 @@ void biff_drag_end(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
 
   mascot=(typMascot *)gdata; 
 
-  gdk_window_raise(win_main->window);
-  gdk_window_raise(clock_main->window);
-  gdk_window_raise(balloon_main->window);
-  gdk_window_raise(biff_pix->window);
+  raise_all();
   
   if (mascot->mail.drag){
     mascot->mail.drag = FALSE;
@@ -695,7 +889,7 @@ void biff_drag_end(GtkWidget * widget, GdkEventButton * event, gpointer gdata)
   gdk_pointer_ungrab(GDK_CURRENT_TIME);
   gdk_flush();
 }
-
+#endif
 
 
 // Mascot WindowѷˤȤʤCallback
@@ -710,10 +904,18 @@ gint dw_configure_main(GtkWidget *widget, GdkEventConfigure *event,
 #endif
 
   if (pixmap_main[0]) {
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(pixmap_main[0]));
+#else
     gdk_pixmap_unref(pixmap_main[0]);
+#endif
   } 
   if (pixmap_main[1]) {
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(pixmap_main[1]));
+#else
     gdk_pixmap_unref(pixmap_main[1]);
+#endif
   } 
   
   pixmap_main[0] = gdk_pixmap_new(widget->window,
@@ -725,21 +927,29 @@ gint dw_configure_main(GtkWidget *widget, GdkEventConfigure *event,
 			       widget->allocation.height,
 			       -1);
   if(mascot->gc_main[0]==NULL){
-    mascot->gc_main[0] = CreatePen(pixmap_main[0], mascot->colclk);
-    mascot->gc_mainsd[0] = CreatePen(pixmap_main[0], mascot->colclksd);
+    mascot->gc_main[0] = MPCreatePen(pixmap_main[0], mascot->colclk);
+    mascot->gc_mainsd[0] = MPCreatePen(pixmap_main[0], mascot->colclksd);
     if(mascot->gc_main[1]==NULL){
-      mascot->gc_main[1] = CreatePen(pixmap_main[1], mascot->colclk);
-      mascot->gc_mainsd[1] = CreatePen(pixmap_main[1], mascot->colclksd);
+      mascot->gc_main[1] = MPCreatePen(pixmap_main[1], mascot->colclk);
+      mascot->gc_mainsd[1] = MPCreatePen(pixmap_main[1], mascot->colclksd);
     }
+#ifndef USE_WIN32
     if(errflag==-1){
       XSetErrorHandler (ehandler);
     }
+#endif
   }
   DrawMascot0(mascot);
+
+#ifdef USE_WIN32  // Get Handle of Task-Bar
+  hWndTaskBar = FindWindow("Shell_TrayWnd", NULL);
+#endif  
+
   return TRUE;
 }
 
 
+#ifdef USE_BIFF
 // Biff Pixmap  WindowѷˤȤʤCallback
 gint dw_configure_biff_pix(GtkWidget *widget, GdkEventConfigure *event,
 			   gpointer gdata)
@@ -747,7 +957,11 @@ gint dw_configure_biff_pix(GtkWidget *widget, GdkEventConfigure *event,
   typMascot *mascot;
   mascot=(typMascot *)gdata; 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(widget->window,
+#else
   gdk_draw_pixmap(widget->window,
+#endif
 		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
 		  mascot->mail.pixmap,
 		  0,0,0,0,
@@ -755,7 +969,7 @@ gint dw_configure_biff_pix(GtkWidget *widget, GdkEventConfigure *event,
 		  widget->allocation.height);
   return TRUE;
 }
-
+#endif
 
 // Clock PanelѷˤȤʤCallback
 gint dw_configure_clk(GtkWidget *widget, GdkEventConfigure *event,
@@ -764,47 +978,47 @@ gint dw_configure_clk(GtkWidget *widget, GdkEventConfigure *event,
   typMascot *mascot;
   mascot=(typMascot *)gdata; 
 
-  if(pixmap_clk[0]==NULL){
+  if (!pixmap_clk[0]) {
     pixmap_clk[0] = gdk_pixmap_new(widget->window,
-				widget->allocation.width,
-				widget->allocation.height,
-				-1);
+				   widget->allocation.width,
+				   widget->allocation.height,
+				   -1);
+  } 
+  if (!pixmap_clk[1]) {
+    pixmap_clk[1] = gdk_pixmap_new(widget->window,
+				   widget->allocation.width,
+				   widget->allocation.height,
+				   -1);
   }
 
-  if(pixmap_clk[1]==NULL){
-    pixmap_clk[1] = gdk_pixmap_new(widget->window,
-				widget->allocation.width,
-				widget->allocation.height,
-				-1);
-  }
+
 
   if(mascot->gc_clk[0]==NULL){
-    mascot->gc_clk[0] = CreatePen(pixmap_clk[0], mascot->colclk);
-    mascot->gc_clksd[0] = CreatePen(pixmap_clk[0], mascot->colclksd);
-    mascot->gc_clkbg[0] = CreatePen(pixmap_clk[0], mascot->colclkbg);
-    mascot->gc_clkbd[0] = CreatePen(pixmap_clk[0], mascot->colclkbd);
+    mascot->gc_clk[0] = MPCreatePen(pixmap_clk[0], mascot->colclk);
+    mascot->gc_clksd[0] = MPCreatePen(pixmap_clk[0], mascot->colclksd);
+    mascot->gc_clkbg[0] = MPCreatePen(pixmap_clk[0], mascot->colclkbg);
+    mascot->gc_clkbd[0] = MPCreatePen(pixmap_clk[0], mascot->colclkbd);
   }
   if(mascot->gc_clk[1]==NULL){
-    mascot->gc_clk[1] = CreatePen(pixmap_clk[1], mascot->colclk);
-    mascot->gc_clksd[1] = CreatePen(pixmap_clk[1], mascot->colclksd);
-    mascot->gc_clkbg[1] = CreatePen(pixmap_clk[1], mascot->colclkbg);
-    mascot->gc_clkbd[1] = CreatePen(pixmap_clk[1], mascot->colclkbd);
+    mascot->gc_clk[1] = MPCreatePen(pixmap_clk[1], mascot->colclk);
+    mascot->gc_clksd[1] = MPCreatePen(pixmap_clk[1], mascot->colclksd);
+    mascot->gc_clkbg[1] = MPCreatePen(pixmap_clk[1], mascot->colclkbg);
+    mascot->gc_clkbd[1] = MPCreatePen(pixmap_clk[1], mascot->colclkbd);
   }
 
-
   if(mascot->clkmode==CLOCK_PANEL){
     // **** Mascot windowؤ
     // **** windowؤ
+#ifdef USE_WIN32
+    if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
+      gdk_window_set_back_pixmap(mascot->w_clkdraw_fg->window,
+				 pixmap_clk[mascot->clk_page],
+				 FALSE);
+    }
+#endif
     gdk_window_set_back_pixmap(mascot->w_clkdraw->window,
-			       pixmap_clk[mascot->clk_page],
+    			       pixmap_clk[mascot->clk_page],
 			       FALSE);
-    //gdk_draw_pixmap(mascot->w_clkdraw->window,
-    //		    mascot->w_clkdraw->style->fg_gc[GTK_WIDGET_STATE (mascot->w_clkdraw)],
-    //		    pixmap_clk[mascot->clk_page],
-    //		    0,0,0,0,
-    //		    widget->allocation.width,
-    //		    widget->allocation.height);
-
   }
   return TRUE;
 }
@@ -833,14 +1047,14 @@ gint dw_configure_bal(GtkWidget *widget, GdkEventConfigure *event,
   }
 
   if(mascot->gc_bal[0]==NULL){
-    mascot->gc_bal[0] = CreatePen(pixmap_bal[0], mascot->colbal);
-    mascot->gc_balbg[0] = CreatePen(pixmap_bal[0], mascot->colbalbg);
-    mascot->gc_balbd[0] = CreatePen(pixmap_bal[0], mascot->colbalbd);
+    mascot->gc_bal[0] = MPCreatePen(pixmap_bal[0], mascot->colbal);
+    mascot->gc_balbg[0] = MPCreatePen(pixmap_bal[0], mascot->colbalbg);
+    mascot->gc_balbd[0] = MPCreatePen(pixmap_bal[0], mascot->colbalbd);
   }
   if(mascot->gc_bal[1]==NULL){
-    mascot->gc_bal[1] = CreatePen(pixmap_bal[1], mascot->colbal);
-    mascot->gc_balbg[1] = CreatePen(pixmap_bal[1], mascot->colbalbg);
-    mascot->gc_balbd[1] = CreatePen(pixmap_bal[1], mascot->colbalbd);
+    mascot->gc_bal[1] = MPCreatePen(pixmap_bal[1], mascot->colbal);
+    mascot->gc_balbg[1] = MPCreatePen(pixmap_bal[1], mascot->colbalbg);
+    mascot->gc_balbd[1] = MPCreatePen(pixmap_bal[1], mascot->colbalbd);
   }
   return TRUE;
 }
@@ -858,7 +1072,11 @@ gint dw_expose_main(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
   mascot=(typMascot *)gdata; 
   
   // **** Mascot windowؤ
+#ifdef USE_GTK2
+  gdk_draw_drawable(widget->window,
+#else
   gdk_draw_pixmap(widget->window,
+#endif
 		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		  pixmap_main[mascot->pixmap_page],
 		  event->area.x, event->area.y,
@@ -872,6 +1090,7 @@ gint dw_expose_main(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
 }
 
 
+#ifdef USE_BIFF
 // Biff Pixmap Ťʤν
 gint dw_expose_biff_pix(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
 {
@@ -880,7 +1099,11 @@ gint dw_expose_biff_pix(GtkWidget *widget, GdkEventExpose *event,  gpointer gdat
   mascot=(typMascot *)gdata; 
 
   if(mascot->mail.flag){
+#ifdef USE_GTK2
+    gdk_draw_drawable(widget->window,
+#else
     gdk_draw_pixmap(widget->window,
+#endif
 		    widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
 		    mascot->mail.pixmap,
 		    0,0,0,0,
@@ -890,6 +1113,7 @@ gint dw_expose_biff_pix(GtkWidget *widget, GdkEventExpose *event,  gpointer gdat
 
   return FALSE;
 }
+#endif
 
 
 // Clock Panel Ťʤν
@@ -903,8 +1127,12 @@ gint dw_expose_clk(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
   if(mascot->clkmode==CLOCK_PANEL){
     // **** Mascot windowؤ
     // **** windowؤ
-    gdk_draw_pixmap(mascot->w_clkdraw->window,
-		    mascot->w_clkdraw->style->fg_gc[GTK_WIDGET_STATE (mascot->w_clkdraw)],
+#ifdef USE_GTK2
+    gdk_draw_drawable(widget->window,
+#else
+    gdk_draw_pixmap(widget->window,
+#endif
+		    mascot->w_clkdraw->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		    pixmap_clk[mascot->clk_page],
 		    0,0,0,0,
 		    widget->allocation.width,
@@ -918,27 +1146,25 @@ gint dw_expose_clk(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
 
 // Balloon Ťʤν
 // WorkƤʤBufferPixmapԤ
+// 1.6.1Ǽ¼ drawballoonΤʤ back_pixmapꤹΤߤȤ
 gint dw_expose_bal(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
 {
   typMascot *mascot;
   mascot=(typMascot *)gdata; 
-
+  
   // **** Balloon windowؤ
-  gdk_window_set_back_pixmap(widget->window,
-			     pixmap_bal[mascot->bal_page],
-			     FALSE);
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(widget->window,
+#else
   gdk_draw_pixmap(widget->window,
+#endif
 		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		  pixmap_bal[mascot->bal_page],
 		  0,0,0,0,
 		  widget->allocation.width,
 		  widget->allocation.height);
-  gtk_widget_shape_combine_mask( balloon_main,
-				 mask_bal[mascot->bal_page],
-				 0,0);
-
-
+  
   return FALSE;
 }
 
@@ -1001,7 +1227,7 @@ void clk_window_motion(GtkWidget * widget, GdkEventMotion * event, gpointer gdat
     gint mx, my, newx, newy;
     GdkModifierType modmask;
     
-    gdk_window_set_cursor(clock_main->window,mascot->cursor.drag_f);
+    gdk_window_set_cursor(widget->window,mascot->cursor.drag_f);
     gdk_window_get_pointer(NULL, &mx, &my, &modmask);
     newx = mx - clk_window_x;
     newy = my - clk_window_y;
@@ -1013,6 +1239,7 @@ void clk_window_motion(GtkWidget * widget, GdkEventMotion * event, gpointer gdat
 }   
 
 
+#ifdef USE_BIFF
 // Biff ɥåˤư
 void biff_window_motion(GtkWidget * widget, GdkEventMotion * event, gpointer gdata)
 {   
@@ -1038,6 +1265,7 @@ void biff_window_motion(GtkWidget * widget, GdkEventMotion * event, gpointer gda
     MoveBiffPix(mascot, mascot->x, mascot->y);
   }
 }   
+#endif
 
 
 void focus_in(GtkWidget * widget, GdkEventMotion * event)
@@ -1068,21 +1296,34 @@ gint time_update(gpointer gdata)
   flag_clock=FALSE;
   mascot=(typMascot *)gdata; 
 
+  // ˥塼ФƤ
   if(mascot->flag_menu) return;
 
+  // Window Sitter
   if(mascot->move==MOVE_FOCUS){
     MoveToFocus(mascot, FALSE);
   }
+  // ޥå
   if(mascot->move==MOVE_FIX){
     mascot->xfix=mascot->x;
     mascot->yfix=mascot->y;
   }
 
+#ifdef USE_BIFF
+  // POP biff
+  if(mascot->mail.pop_readed==TRUE){
+      display_biff_balloon(mascot);
+      mascot->mail.pop_readed=FALSE;
+  }
+#endif
 
+  // դ
   if(flag_balloon==TRUE){
     DoBalloon(mascot);
   }
 
+
+#ifdef USE_SOCKMSG
   // Duet˥
   if(mascot->duet_seq>0){
     mascot->duet_seq--;
@@ -1118,7 +1359,7 @@ gint time_update(gpointer gdata)
       }
     }
   }
-
+#endif  // USE_SOCKMSG
 
   // ˥
   mascot->anime_seq++;
@@ -1188,6 +1429,9 @@ gint time_update(gpointer gdata)
 	    ||(mascot->bal_mode==BALLOON_DUET))){
 	  // Х롼󤬤ǤƤĤ
 	  // å˥νλ
+#ifdef USE_WIN32
+	    gtk_widget_unmap(balloon_fg);
+#endif
 	    gtk_widget_unmap(balloon_main);
 	    flag_balloon=FALSE;
 	}
@@ -1235,6 +1479,8 @@ gint time_update(gpointer gdata)
     printf("Draw finish %d\n",work_page);
 #endif
   }
+
+  if(mascot->raise_force) raise_all();
     
   gdk_flush();
 
@@ -1245,18 +1491,22 @@ gint time_update(gpointer gdata)
 void clock_update(typMascot *mascot, gboolean force_flag){
   time_t t;
   struct tm *tmpt;
-  static char caldigit[20];
-  gchar *weekk;
+  gchar* caldigit;
 
   t = time(NULL);
   tmpt = localtime(&t);
   
   if(tmpt->tm_mday!=old_day){
-    weekk=g_strdup(ini_week(tmpt->tm_wday));
-    sprintf(caldigit,_("%d/%d/%d(%s)"),
-	    tmpt->tm_year+1900,tmpt->tm_mon+1,tmpt->tm_mday,weekk);
+
+    caldigit=g_strdup_printf(_("%d/%d/%d(%s)"), tmpt->tm_year+1900,tmpt->tm_mon+1,
+                                      tmpt->tm_mday,ini_week(tmpt->tm_wday));
 
     gtk_tooltips_set_tip(tooltip,clock_main,caldigit,NULL);
+#ifdef USE_WIN32
+    gtk_tooltips_set_tip(tooltip,clock_fg,caldigit,NULL);
+#endif
+
+    g_free(caldigit);
     old_day=tmpt->tm_mday;
   }
 
@@ -1405,6 +1655,7 @@ void DrawPanelClock(typMascot *mascot)
 #else
   gint lb,rb,as,ds;
 #endif
+  gint new_w, new_h;
 
   work_page=mascot->clk_page;
   work_page^=1;
@@ -1427,18 +1678,21 @@ void DrawPanelClock(typMascot *mascot)
     break;
   }
 
-  gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_clkdraw), 
-			 clk_width+(mascot->clktext_x+mascot->wclkbd)*2,
-			 clk_height+(mascot->clktext_y+mascot->wclkbd)*2);
   
   if (pixmap_clk[work_page]) {
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(pixmap_clk[work_page]));
+#else
     gdk_pixmap_unref(pixmap_clk[work_page]);
+#endif
   } 
   
+  new_w=clk_width+(mascot->clktext_x+mascot->wclkbd)*2;
+  new_h=clk_height+(mascot->clktext_y+mascot->wclkbd)*2;
   pixmap_clk[work_page] = gdk_pixmap_new(mascot->w_clkdraw->window,
-			       mascot->w_clkdraw->allocation.width,
-			       mascot->w_clkdraw->allocation.height,
-			       -1);
+					 new_w,
+					 new_h,
+					 -1);
   
 
   // **** pixmapؤBUFFERING
@@ -1530,14 +1784,53 @@ void DrawPanelClock(typMascot *mascot)
     break;
   }
 
-
-  gdk_window_resize (clock_main->window,
-		     clk_width+(mascot->clktext_x+mascot->wclkbd)*2,
-		     clk_height+(mascot->clktext_y+mascot->wclkbd)*2);
-
   mascot->clk_page=work_page;
+  gdk_flush();
 
-  while (g_main_iteration(FALSE));
+
+#ifdef USE_WIN32
+  if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
+    gdk_window_set_back_pixmap(mascot->w_clkdraw_fg->window,
+			       pixmap_clk[mascot->clk_page],
+			       FALSE);
+  }
+#endif
+  gdk_window_set_back_pixmap(mascot->w_clkdraw->window,
+			     pixmap_clk[mascot->clk_page],
+			     FALSE);
+
+  if((mascot->w_clkdraw->allocation.width!=new_w)
+     ||(mascot->w_clkdraw->allocation.width!=new_h)){
+#ifdef USE_WIN32
+    if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
+      gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_clkdraw_fg), 
+			     new_w,
+			     new_h);
+    }
+#endif
+
+    gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_clkdraw), 
+			   new_w,
+			   new_h);
+
+#ifdef USE_WIN32
+    if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
+      gdk_window_resize (clock_fg->window,
+			 new_w,
+			 new_h);
+    }
+#endif
+    gdk_window_resize (clock_main->window,
+		       new_w,
+		       new_h);
+  }
+
+
+#ifdef USE_GTK2
+  g_object_unref(G_OBJECT(pango_text));
+#endif
+
+  while (my_main_iteration(FALSE));
   gdk_flush();
 }
 
@@ -1575,6 +1868,24 @@ gchar *ini_week(gint day_of_week){
 // Ǥեޥɼ¹
 void ext_play(typMascot *mascot, char *exe_command)
 {
+#ifdef USE_WIN32
+  STARTUPINFO si;
+  PROCESS_INFORMATION pi;
+
+  if(exe_command){
+    ZeroMemory(&si, sizeof(si));
+    si.cb=sizeof(si);
+
+    if(CreateProcess(NULL, (LPTSTR)exe_command, NULL, NULL,
+		     FALSE, NORMAL_PRIORITY_CLASS,
+		     NULL, NULL, &si, &pi)){
+      CloseHandle(pi.hThread);
+      CloseHandle(pi.hProcess);
+    }
+      
+  }
+  
+#else
   static pid_t pid;
   gchar *cmdline;
   
@@ -1587,14 +1898,26 @@ void ext_play(typMascot *mascot, char *exe_command)
 	_exit(-1);
 	signal(SIGCHLD,ChildTerm);
       }
+      g_free(cmdline);
     }
   }
+#endif // USE_WIN32
 }
 
 
 //  (1Ĥ)
 void sound_play(typMascot *mascot, char *wav_name)
 {
+#ifdef USE_WIN32
+  gchar *filename1;
+
+  if( wav_name ){
+
+    if(!(filename1=FullPathSoundFile(mascot, wav_name, FALSE))) return;
+
+    PlaySound(filename1,NULL, SND_ASYNC);
+  }
+#else
   static pid_t pid;
   gchar cmdline[1024];
   gchar *filename1;
@@ -1602,7 +1925,7 @@ void sound_play(typMascot *mascot, char *wav_name)
 
   if( (mascot->sound_command) && (wav_name) ){
 
-    if(!(filename1=FullPathSoundFile(mascot, wav_name))) return;
+    if(!(filename1=FullPathSoundFile(mascot, wav_name, FALSE))) return;
 
     sprintf(cmdline,mascot->sound_command,filename1);
     waitpid(pid,0,WNOHANG);
@@ -1612,9 +1935,11 @@ void sound_play(typMascot *mascot, char *wav_name)
       signal(SIGCHLD,ChildTerm);
     }
   }
+#endif // USE_WIN32
 }
 
 
+#ifndef USE_WIN32
 // ҥץν 
 void ChildTerm(int dummy)
 {
@@ -1623,5 +1948,115 @@ void ChildTerm(int dummy)
   wait(&s);
   signal(SIGCHLD,ChildTerm);
 }
+#endif // USE_WIN32
 
 
+// Raise all window
+void raise_all(void){
+#ifdef USE_WIN32
+  HWND   hWnd;
+
+  SetWindowPos(GDK_WINDOW_HWND(win_main->window),HWND_TOPMOST,0,0,0,0,
+	SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOSIZE);
+  SetWindowLong(GDK_WINDOW_HWND(win_main->window), GWL_EXSTYLE, 
+	GetWindowLong(GDK_WINDOW_HWND(win_main->window), GWL_EXSTYLE)|WS_EX_NOACTIVATE);
+#else
+  gdk_window_raise(win_main->window);
+#endif
+#ifdef USE_WIN32
+  gdk_window_raise(clock_fg->window);
+#endif
+  gdk_window_raise(clock_main->window);
+#ifdef USE_WIN32
+  gdk_window_raise(balloon_fg->window);
+#endif
+  gdk_window_raise(balloon_main->window);
+#ifdef USE_BIFF
+  gdk_window_raise(biff_pix->window);
+#endif
+}
+
+
+void callbacks_arg_init(){
+  pre_hour=-1;
+  pre_min=-1;
+  pre_sec=-1;
+
+  start_x=0;
+  start_y=0;
+#ifdef USE_BIFF
+  biff_start_x=0;
+  biff_start_y=0;
+#endif
+  clk_start_x=0;
+  clk_start_y=0;
+
+  flag_balloon=FALSE;
+  flag_anime=FALSE;
+  flag_clock=FALSE;
+
+  old_day=0;
+  errflag=-1;
+}
+
+#ifdef USE_WIN32
+//*********************************************************
+//  Сΰζ֤
+//*********************************************************
+int GetTaskbarHeight( void )
+{
+  APPBARDATA abd;
+  HWND hWndAutoHide;
+  
+  memset( &abd, 0, sizeof( abd ) );
+  abd.cbSize = sizeof( abd );
+  
+  if( SHAppBarMessage(ABM_GETSTATE, &abd) & ABS_AUTOHIDE ){
+    SHAppBarMessage(ABM_GETAUTOHIDEBAR, &abd);
+  }
+  else{
+    SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
+  }
+  if(abd.uEdge==ABE_BOTTOM){
+    return (abd.rc.bottom - abd.rc.top);
+  }
+  else{
+    return(0);
+  }	
+  
+}//GetTaskbarRect
+
+int GetTaskbarWidth( void )
+{
+  APPBARDATA abd;
+  
+  memset( &abd, 0, sizeof( abd ) );
+  abd.cbSize = sizeof( abd );
+  
+  SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
+  if(abd.uEdge==ABE_BOTTOM){
+    return (abd.rc.right);
+  }
+  else{
+    return(GetSystemMetrics(SM_CXVIRTUALSCREEN));
+  }	
+  
+}//GetTaskbarRect
+
+//*********************************************************
+// Win32 롼ȤȽ̥ɥ֤
+//*********************************************************
+gboolean IsWin32Root( HWND hWnd, gint root_width, gint root_height )
+{
+  RECT Rect;
+  
+  GetWindowRect( hWnd, &Rect ); // ɥμ
+  if((Rect.right - Rect.left)==root_width){
+    if((Rect.bottom - Rect.top)==root_height){
+      return(TRUE);
+    }
+  };
+  return(FALSE);
+}//GetWindowWidth
+#endif
+
