@@ -1,12 +1,13 @@
 //  MaCoPiX = Mascot Construnctive Pilot for X
 //                                (ActX / Gtk+ Evolution)
-//    A unified desktop mascot program
-//    for UNIX / X Window System with Gdk Environment
 //
-//  nokkari.c  : Nokkari-Chara (an Window Sitter for MS Win) Converter
-//                                                         for MaCoPiX
+//
+//      nokkari.c  
+//      Nokkari-Chara (an Window Sitter for MS Win) Converter
 //                   (for Nokkari-chara ver1.40 or later)
-//                                copyright K.Chimari 2002-4
+//
+//                            Copyright 2002-2007  K.Chimari
+//                                     http://rosegray.sakura.ne.jp/
 //
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -21,20 +22,28 @@
 // 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-//
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 //
 
+
 #include "main.h"
 #include "nokkari.h"
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #include <signal.h>
 
 #ifndef USE_GTK2
 #include <png.h>
 #endif
 
-#undef CODE_TEST
+#if HAVE_ICONV
+#  include <iconv.h>
+#endif
+
+#define CODE_TEST
+
+#define BUFFSIZE 4096
 
 void NkrSave();
 void NkrRead();
@@ -45,18 +54,35 @@ void remove_anim_loop();
 void WritePNG();
 #endif
 
+#if HAVE_ICONV
 void CodeHenkanSJIS();
 void CodeHenkanEUC();
+#endif
 
 void create_nkr_dialog();
 static void close_nkr();
 
+extern gchar* to_locale();
+extern gchar* to_utf8();
+
 extern int WriteBMP ();
 
 extern gchar* create_nkr_change_image_dialog();
 
 extern void copy_file();
 
+#if HAVE_ICONV
+extern gchar *conv_iconv_strdup();
+#endif
+
+#ifdef __GTK_STOCK_H__
+extern GtkWidget* gtkut_button_new_from_stock();
+#endif
+
+extern void my_signal_connect();
+extern gchar* my_dirname();
+extern gchar* my_basename();
+extern gboolean my_main_iteration();
 
 int total_ptn;
 gchar *rdm_ptn, *clk_ptn;
@@ -96,7 +122,7 @@ void NkrSave (typMascot *mascot)
   for(i_pix=0;i_pix<mascot->nPixmap;i_pix++){
     if(mascot->sprites[i_pix].filename){
       sprintf(tmp, "File%d", i_pix+1);
-      tmp2=g_strdup(g_basename(mascot->sprites[i_pix].filename));
+      tmp2=g_strdup(my_basename(mascot->sprites[i_pix].filename));
       p=(char *)strstr(tmp2,".png");
       strcpy(p,".bmp");
       
@@ -110,9 +136,13 @@ void NkrSave (typMascot *mascot)
 	g_print (_("Cannot Load Image %s\n"), mascot->sprites[i_pix].filename);
 	exit(1);
       }
-      tmp1=g_strconcat(g_dirname(mascot->inifile),"/",tmp2,NULL);
+      tmp1=g_strconcat(my_dirname(mascot->inifile),G_DIR_SEPARATOR_S,tmp2,NULL);
       WriteBMP (tmp1, pixbuf);
+#ifdef USE_GTK2
+      g_object_unref(G_OBJECT(pixbuf));
+#else
       gdk_pixbuf_unref(pixbuf);
+#endif
       
       if(mascot->sprites[i_pix].filename){
 	xmms_cfg_write_string(cfgfile, "Pat", tmp, tmp2);
@@ -177,31 +207,14 @@ void NkrSave (typMascot *mascot)
     
   fclose(fp);
 
+#if HAVE_ICONV
   CodeHenkanSJIS(mascot);
+#endif
 
   create_nkr_dialog(mascot);
 }
 
 
-/*
-gchar* delspace(gchar *input_char)
-{
-  int i,i_del=0;
-  gchar output[200];
-  gchar *op=NULL;
-
-  for(i=0;i<strlen(input_char);i++){
-    if(input_char[i]!=0x20){
-      output[i_del]=input_char[i];
-      i_del++;
-    }
-  }
-  output[i_del]=0x00;
-
-  op=g_strdup(output);
-  return(op);
-}
-*/
 
 // ˥᡼loopκƸ 1ptn֤
 gint anim_loop(typMascot *mascot, int i_ptn, ConfigFile *cfgfile)
@@ -456,6 +469,7 @@ void NkrRead(typMascot *mascot)
   gchar *p, *wp;
   gint cur_frm,old_frm;
   GdkPixbuf *pixbuf = NULL, *pixbuf2 = NULL;
+  gchar *tmp_open;
   
   nkr_msg=NULL;
   mascot->random_total=0;
@@ -465,11 +479,13 @@ void NkrRead(typMascot *mascot)
   // Iniեľdir
   inifilename = g_strdup(mascot->inifile);
   if(access(inifilename,F_OK)!=0){
-    g_message(_("Cannot Find Mascot File %s\n"),g_basename(mascot->inifile));
+    g_message(_("Cannot Find Mascot File %s\n"),my_basename(mascot->inifile));
     exit(1);
   }
 
+#if HAVE_ICONV
   CodeHenkanEUC(mascot);
+#endif
 
   /*
   mascot->file=g_strdup(filename);
@@ -544,13 +560,14 @@ void NkrRead(typMascot *mascot)
     mascot->colbalbd->green=COLOR_BALBD_G;
     mascot->colbalbd->blue=COLOR_BALBD_B;
 
+#ifdef USE_BIFF
     // Biffѥǡ
     mascot->mail.pix_file=NULL;
     mascot->mail.pix_pos=MAIL_PIX_RIGHT;
     mascot->mail.pix_x=0;
     mascot->mail.pix_y=0;
     mascot->mail.sound=NULL;
-
+#endif
 
 
     // Pixmapǡ
@@ -563,31 +580,38 @@ void NkrRead(typMascot *mascot)
 	break;
       }
       else{//ѥ󵭽Ҥä
-	filename0=g_strconcat(g_dirname(mascot->inifile),"/",tmp1,NULL);
-	if(strlen(g_basename(tmp1))!=0){
+	filename0=g_strconcat(my_dirname(mascot->inifile),G_DIR_SEPARATOR_S,tmp1,NULL);
+	if(strlen(my_basename(tmp1))!=0){
 	  if(access(filename0,F_OK)!=0){
 	    while(!(filename0=
 		    g_strdup(create_nkr_change_image_dialog(tmp1,i_pix)))){
 	    }
 	  }
+	  tmp_open=to_utf8(filename0);
 #ifdef USE_GTK2
-	  pixbuf = gdk_pixbuf_new_from_file(filename0,NULL);
+	  pixbuf = gdk_pixbuf_new_from_file(tmp_open,NULL);
 #else
-	  pixbuf = gdk_pixbuf_new_from_file(filename0);
+	  pixbuf = gdk_pixbuf_new_from_file(tmp_open);
 #endif
 	  // BMPФAlphaȴʬ
 	  pixbuf2=gdk_pixbuf_add_alpha(pixbuf,TRUE,0x00,0xFF,0x00);
-	  tmp2=g_strdup(g_basename(tmp1));
+	  tmp2=g_strdup(my_basename(tmp1));
 	  p=(char *)strstr(tmp2,".bmp");
 	  strcpy(p,".png");
-	  filename0=g_strconcat(g_dirname(mascot->inifile),"/",tmp2,NULL);
+	  filename0=g_strconcat(my_dirname(mascot->inifile),G_DIR_SEPARATOR_S,tmp2,NULL);
 #ifdef USE_GTK2	  
 	  gdk_pixbuf_save(pixbuf2,filename0,"png",NULL,NULL);
 #else
 	  WritePNG(pixbuf2,filename0);
 #endif
+#ifdef USE_GTK2
+	  g_object_unref(G_OBJECT(pixbuf));
+	  g_object_unref(G_OBJECT(pixbuf2));
+#else
 	  gdk_pixbuf_unref(pixbuf);
 	  gdk_pixbuf_unref(pixbuf2);
+#endif
+	  g_free(tmp_open);
 
 	  mascot->sprites[i_pix].filename=
 	    (gchar *)FullPathPixmapFile(mascot, filename0);
@@ -741,124 +765,115 @@ void NkrRead(typMascot *mascot)
   g_free(tmp2);
 }
 
-
+#if HAVE_ICONV
 
 void CodeHenkanSJIS(typMascot *mascot){
-  char systemp[512];
   gchar *tmpname;
-#ifdef CODE_TEST
   FILE *src_fp, *dest_fp;
-  gchar *buf;
-  gint n_read;
-  gchar *res;
-  static GIConv converter;
-  static gboolean converter_open = FALSE;
+  gchar *buf, *buf2, *buf3=NULL;
 
-  if (!converter_open) {
-    converter_open = TRUE;
-    converter = g_iconv_open (NKR_CODE,"UTF-8");
-  }
-  
-  buf=g_malloc0(sizeof(gchar)*1024);
+  buf=g_malloc0(sizeof(gchar)*BUFFSIZE);
 
-  tmpname=g_strdup("tmpcode.XXXXXX");
+#ifdef USE_WIN32
+  tmpname = g_strdup_printf("%s%smacopix-tmpcode.XXXXXX",
+			    g_get_tmp_dir(), G_DIR_SEPARATOR_S);
+#else
+  tmpname = g_strdup_printf("%s%smacopix-tmpcode.%d.XXXXXX",
+			    g_get_tmp_dir(), G_DIR_SEPARATOR_S,
+			    getuid());
+#endif
 
   if ((src_fp = fopen(mascot->inifile, "rb")) == NULL) {
     g_print(_("Cannot open copy source file %s"),mascot->inifile);
     exit(1);
   }
 
-  if ((dest_fp = fopen(mkstemp(tmpname), "wb")) == NULL) {
+  if ((dest_fp = fopen(tmpname, "wb")) == NULL) {
     g_print(_("Cannot open copy destination file %s"),tmpname);
     exit(1);
   }
 
-  while (!feof(src_fp)){
-    n_read = fread(buf, sizeof(gchar), 1024, src_fp);
-    res = g_convert_with_iconv (buf, n_read*sizeof(gchar), converter, NULL, NULL, NULL);
-    if(res) fwrite(res, strlen(res), sizeof(gchar), dest_fp);
+
+  while( fgets( buf,BUFFSIZE-2,src_fp ) != NULL ){
+
+#ifdef USE_GTK2
+    buf2=conv_iconv_strdup(buf, "UTF-8", "SJIS");
+#else
+    buf2=conv_iconv_strdup(buf, "EUCJP", "SJIS");
+#endif
+    if(buf2[strlen(buf2)-1]=='\n'){
+      buf2[strlen(buf2)-1]='\r';
+      buf3=g_strconcat(buf2,"\n",NULL);
+    }
+    fwrite(buf3, strlen(buf3), sizeof(gchar), dest_fp);
+    g_free(buf2);
+    if(buf3) g_free(buf3);
   }
+
   fclose(dest_fp);
   fclose(src_fp);
 
+#ifndef USE_WIN32
   chmod(tmpname, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+#endif
 
   g_free(buf);
 
-#else
-
-  tmpname = g_strdup_printf("%s%cmacopix-tmpcode.%d.XXXXXX",
-			    g_get_tmp_dir(), G_DIR_SEPARATOR,
-			    getuid());
-  mkstemp(tmpname);
-  sprintf(systemp,"%s %s > %s", mascot->sjis_filter,
-  	  mascot->inifile, tmpname);
-  system(systemp);
-
-#endif
-
   copy_file(tmpname,mascot->inifile);
   unlink(tmpname);
 }
 
 void CodeHenkanEUC(typMascot *mascot){
-  char systemp[512];
   gchar *tmpname;
-#ifdef CODE_TEST
   FILE *src_fp, *dest_fp;
-  gchar *buf;
-  gint n_read;
-  gchar *res;
-  static GIConv converter;
-  static gboolean converter_open = FALSE;
+  gchar *buf, *buf2;
 
-  if (!converter_open) {
-    converter_open = TRUE;
-    converter = g_iconv_open ("UTF-8",NKR_CODE);
-  }
-  
-  buf=g_malloc0(sizeof(gchar)*1024);
+  buf=g_malloc0(sizeof(gchar)*BUFFSIZE);
 
-  tmpname=g_strdup("tmpcode.XXXXXX");
+#ifdef USE_WIN32
+  tmpname = g_strdup_printf("%s%smacopix-tmpcode.XXXXXX",
+			    g_get_tmp_dir(), G_DIR_SEPARATOR_S);
+#else
+  tmpname = g_strdup_printf("%s%smacopix-tmpcode.%d.XXXXXX",
+			    g_get_tmp_dir(), G_DIR_SEPARATOR_S,
+			    getuid());
+#endif
 
   if ((src_fp = fopen(mascot->inifile, "rb")) == NULL) {
     g_print(_("Cannot open copy source file %s"),mascot->inifile);
     exit(1);
   }
 
-  if ((dest_fp = fopen(mkstemp(tmpname), "wb")) == NULL) {
+  if ((dest_fp = fopen(tmpname, "wb")) == NULL) {
     g_print(_("Cannot open copy destination file %s"),tmpname);
     exit(1);
   }
 
-  while (!feof(src_fp)){
-    n_read = fread(buf, sizeof(gchar), 1024, src_fp);
-    res = g_convert_with_iconv (buf, n_read*sizeof(gchar), converter, NULL, NULL, NULL);
-    if(res) fwrite(res, strlen(res), sizeof(gchar), dest_fp);
+
+  while( fgets( buf,BUFFSIZE-2,src_fp ) != NULL ){
+
+    buf2=conv_iconv_strdup(buf, "SJIS", "EUCJP");
+    if((buf2[strlen(buf2)-2]=='\r')&&(buf2[strlen(buf2)-1]=='\n')){
+      buf2[strlen(buf2)-2]='\n';
+      buf2[strlen(buf2)-1]='\0';
+    }
+    fwrite(buf2, strlen(buf2), sizeof(gchar), dest_fp);
+    g_free(buf2);
   }
+
   fclose(dest_fp);
   fclose(src_fp);
 
+#ifndef USE_WIN32
   chmod(tmpname, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+#endif
 
   g_free(buf);
 
-#else
-
-  tmpname = g_strdup_printf("%s%cmacopix-tmpcode.%d.XXXXXX",
-			    g_get_tmp_dir(), G_DIR_SEPARATOR,
-			    getuid());
-  mkstemp(tmpname);
-  sprintf(systemp,"%s %s > %s", mascot->euc_filter,
-	  mascot->inifile, tmpname);
-  system(systemp);
-
-#endif
-
   copy_file(tmpname,mascot->inifile);
   unlink(tmpname);
 }
-
+#endif /* HAVE_ICONV */
 
 #ifndef USE_GTK2
 // From "Linux Moe Moe project"  
@@ -958,7 +973,7 @@ void create_nkr_dialog(typMascot *mascot)
 #endif
 
   // WinۤϽŤΤExpose٥򤹤٤ƽƤ
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
 
   mascot->flag_menu=TRUE;
 
@@ -973,9 +988,7 @@ void create_nkr_dialog(typMascot *mascot)
   gtk_window_set_title(GTK_WINDOW(nkr_main), 
 		       _("MaCoPiX <--> Nokkari-Chara Convert Log"));
   gtk_widget_realize(nkr_main);
-  gtk_signal_connect(GTK_OBJECT(nkr_main),"destroy",
-		     GTK_SIGNAL_FUNC(close_nkr), 
-		     GTK_WIDGET(nkr_main));
+  my_signal_connect(nkr_main,"destroy",close_nkr, GTK_WIDGET(nkr_main));
   gtk_container_set_border_width (GTK_CONTAINER (nkr_main), 5);
   
   // 6x3Υơ֥
@@ -1025,12 +1038,14 @@ void create_nkr_dialog(typMascot *mascot)
 #endif
  
 
+#ifdef __GTK_STOCK_H__
+  button=gtkut_button_new_from_stock(_("OK"),GTK_STOCK_OK);
+#else
   button=gtk_button_new_with_label(_("OK"));
+#endif
   gtk_table_attach(GTK_TABLE(nkr_tbl), button, 4, 5, 2, 3,
 		   GTK_FILL,GTK_SHRINK,0,0);
-  gtk_signal_connect(GTK_OBJECT(button),"pressed",
-		     GTK_SIGNAL_FUNC(close_nkr), 
-		     GTK_WIDGET(nkr_main));
+  my_signal_connect(button,"clicked",close_nkr, GTK_WIDGET(nkr_main));
   
   gtk_widget_show_all(nkr_main);
   
@@ -1042,9 +1057,9 @@ static void close_nkr(GtkWidget *w, GtkWidget *dialog)
 {
   //gdk_pointer_ungrab(GDK_CURRENT_TIME);
 
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
   gtk_widget_destroy(GTK_WIDGET(dialog));
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
  
   Mascot->flag_menu=FALSE;
   gdk_flush();
