@@ -1,11 +1,13 @@
 //  MaCoPiX = Mascot Construnctive Pilot for X
 //                                (ActX / Gtk+ Evolution)
-//    A unified desktop mascot program
-//    for UNIX / X Window System with Gdk Environment
 //
-//  pixmap.c  : Management of Image loading & rendering for MaCoPiX
+//
+//      pixmap.c  
+//      Management of Image loading & rendering for MaCoPiX
 //       (using gdk-pixbuf)
-//                                copyright K.Chimari 2002-4
+//
+//                            Copyright 2002-2007  K.Chimari
+//                                     http://rosegray.sakura.ne.jp/
 //
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,24 +22,47 @@
 // 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-//
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 //
 
 #include "main.h"
 
+#ifdef USE_BIFF
 #include "def_biff.xpm"
 #include "def_biff_r.xpm"
+#endif
+
 
 // *** GLOBAL ARGUMENT ***
-extern GtkWidget *win_main;
+extern GtkWidget *win_main, *clock_main, *balloon_main, *biff_pix;
+#ifdef USE_WIN32
+extern GtkWidget *clock_fg, *balloon_fg;
+#endif
 extern GtkWidget *popup_menu; 
 extern gint window_x, window_y;
 
 extern GdkDrawable *pixmap_main[2], *pixmap_clk[2], *pixmap_bal[2];
 
-extern GdkGC *CreatePen();
+extern GdkGC *MPCreatePen();
 
+extern gchar* to_utf8();
+
+#ifdef __GTK_STATUS_ICON_H__
+extern void trayicon_set_tooltip();
+#endif
+
+#ifdef __GTK_STOCK_H__
+extern GtkWidget* gtkut_button_new_from_stock();
+#endif
+
+extern void my_signal_connect();
+extern gboolean my_main_iteration();
+
+
+#ifdef USE_WIN32
+void GdkWinChangeAlpha();
+void GdkWinChangeAlphaFG();
+#endif
 
 //CallbackϤ碌¤ ǥեȥեѹ
 typedef struct{
@@ -51,7 +76,9 @@ typedef struct{
 // Prototype of Functions in this file
 gboolean TestLoadPixmaps();
 void LoadPixmaps();
+#ifdef USE_BIFF
 void LoadBiffPixmap();
+#endif
 gint DrawMascot0();
 gint DrawMascot();
 gint DrawMascotTemp();
@@ -67,12 +94,16 @@ gboolean TestLoadPixmaps(typMascot *mascot, gchar *filename, gint i_pix)
   GdkPixbuf *pixbuf = NULL,*pixbuf2 = NULL;
   gint w=0,h=0;
   gint ipstyle;
+  gchar *tmp_open;
 
+  
+  tmp_open=to_utf8(filename);
 #ifdef USE_GTK2
-  pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
+  pixbuf = gdk_pixbuf_new_from_file(tmp_open, NULL);
 #else
-  pixbuf = gdk_pixbuf_new_from_file(filename);
+  pixbuf = gdk_pixbuf_new_from_file(tmp_open);
 #endif
+  g_free(tmp_open);
   if(!pixbuf) return(FALSE);
   
   w = gdk_pixbuf_get_width(pixbuf)*((gfloat)(mascot->magnify)/100);
@@ -100,10 +131,27 @@ gboolean TestLoadPixmaps(typMascot *mascot, gchar *filename, gint i_pix)
   }
     
   pixbuf2=gdk_pixbuf_scale_simple(pixbuf,w,h,ipstyle);
+  if(mascot->sprites[i_pix].pixmap) 
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(mascot->sprites[i_pix].pixmap));
+#else
+    gdk_pixmap_unref(mascot->sprites[i_pix].pixmap);
+#endif
+  if(mascot->sprites[i_pix].mask) 
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(mascot->sprites[i_pix].mask));
+#else
+    gdk_pixmap_unref(mascot->sprites[i_pix].mask);
+#endif
   gdk_pixbuf_render_pixmap_and_mask(pixbuf2, &mascot->sprites[i_pix].pixmap,
 				      &mascot->sprites[i_pix].mask, 0xc0);
+#ifdef USE_GTK2
+  g_object_unref(G_OBJECT(pixbuf));
+  g_object_unref(G_OBJECT(pixbuf2));
+#else
   gdk_pixbuf_unref(pixbuf);
   gdk_pixbuf_unref(pixbuf2);
+#endif
 
   DrawMascotTemp(mascot,i_pix);
 
@@ -118,30 +166,55 @@ void LoadPixmaps(GtkWidget *widget, GtkWidget *draw,
   GdkPixbuf *pixbuf = NULL, *pixbuf2 = NULL;
   gint w=0,h=0;
   gint ipstyle;
-  gchar *filename1=NULL, *filename2=NULL;
 #ifndef USE_GTK2
   gchar *fontset=NULL;
 #endif
+  gchar *tmp_open;
+
+#ifdef USE_WIN32
+  GdkWinChangeAlpha(win_main->window, mascot->alpha_main);
+#ifdef USE_BIFF
+  GdkWinChangeAlpha(biff_pix->window, mascot->alpha_biff);
+#endif
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    GdkWinChangeAlphaFG(balloon_fg->window, mascot->colbalbg);
+  }
+  GdkWinChangeAlpha(balloon_main->window, mascot->alpha_bal);
+  if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
+    GdkWinChangeAlphaFG(clock_fg->window, mascot->colclkbg);
+  }
+  GdkWinChangeAlpha(clock_main->window, mascot->alpha_clk);
+#endif
 
-  filename1=g_malloc0(sizeof(gchar)*512);
-  filename2=g_malloc0(sizeof(gchar)*512);
 
   while(sprites[i].filename){
-    if (sprites[i].pixmap) gdk_pixmap_unref(sprites[i].pixmap);
-    if (sprites[i].mask) gdk_pixmap_unref(sprites[i].mask);
-
-    //im = gdk_imlib_load_image(sprites[i].filename);
+    if (sprites[i].pixmap)
 #ifdef USE_GTK2
-    pixbuf = gdk_pixbuf_new_from_file(sprites[i].filename, NULL);
+      g_object_unref(G_OBJECT(sprites[i].pixmap));
 #else
-    pixbuf = gdk_pixbuf_new_from_file(sprites[i].filename);
+      gdk_pixmap_unref(sprites[i].pixmap);
+#endif
+    if (sprites[i].mask)
+#ifdef USE_GTK2
+      g_object_unref(G_OBJECT(sprites[i].mask));
+#else
+      gdk_pixmap_unref(sprites[i].mask);
+#endif
+    //im = gdk_imlib_load_image(sprites[i].filename);
+  tmp_open=to_utf8(sprites[i].filename);
+#ifdef USE_GTK2
+    pixbuf = gdk_pixbuf_new_from_file(tmp_open, NULL);
+#else
+    pixbuf = gdk_pixbuf_new_from_file(tmp_open);
 #endif
-    
     // if(im==NULL){
     if(pixbuf==NULL){
-      g_print (_("Cannot Load Image %s\n"), sprites[i].filename);
+      //g_print (_("Cannot Load Image %s\n"), sprites[i].filename);
+      g_print (_("Cannot Load Image %s\n"), tmp_open);
+      g_free(tmp_open);
       exit(1);
     }
+    g_free(tmp_open);
 
     w = gdk_pixbuf_get_width(pixbuf)*((gfloat)(mascot->magnify)/100);
     h = gdk_pixbuf_get_height(pixbuf)*((gfloat)(mascot->magnify)/100);
@@ -178,8 +251,13 @@ void LoadPixmaps(GtkWidget *widget, GtkWidget *draw,
     gdk_pixbuf_render_pixmap_and_mask(pixbuf2, &sprites[i].pixmap,
 				      &sprites[i].mask, 0xc0);
 
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(pixbuf));
+    g_object_unref(G_OBJECT(pixbuf2));
+#else
     gdk_pixbuf_unref(pixbuf);
     gdk_pixbuf_unref(pixbuf2);
+#endif
     i++;
   }
   mascot->width=w;
@@ -188,9 +266,6 @@ void LoadPixmaps(GtkWidget *widget, GtkWidget *draw,
 			 mascot->width, mascot->height);
   gdk_window_resize (widget->window, w, h);
 
-  g_free(filename1);
-  g_free(filename2);
-
   
   // եȤΥ
 #ifdef USE_GTK2
@@ -287,9 +362,15 @@ void LoadPixmaps(GtkWidget *widget, GtkWidget *draw,
   gtk_widget_shape_combine_mask( win_main, 
 				 mascot->sprites[mascot->frame_pix[0][0]].mask,
 				 0, 0 ); 
+
+#ifdef __GTK_STATUS_ICON_H__
+  trayicon_set_tooltip(mascot);
+#endif
+
 }
 
 
+#ifdef USE_BIFF
 void LoadBiffPixmap(GtkWidget *widget, typMascot *mascot){
   gchar *fontset=NULL;
   int i=0;
@@ -297,6 +378,7 @@ void LoadBiffPixmap(GtkWidget *widget, typMascot *mascot){
   GdkPixbuf *pixbuf = NULL, *pixbuf2 = NULL;
   gint ipstyle;
   gint w=0,h=0;
+  gchar *tmp_open;
 
   switch(mascot->ip_style){
   case MAG_IP_NEAREST:
@@ -321,14 +403,26 @@ void LoadBiffPixmap(GtkWidget *widget, typMascot *mascot){
 
 
   // ᥤpixmap
-  if (mascot->mail.pixmap) gdk_pixmap_unref(mascot->mail.pixmap);
-  if (mascot->mail.mask) gdk_pixmap_unref(mascot->mail.mask);
-  if(mascot->mail.pix_file){
+  if (mascot->mail.pixmap)
 #ifdef USE_GTK2
-    pixbuf = gdk_pixbuf_new_from_file(mascot->mail.pix_file, NULL);
+    g_object_unref(G_OBJECT(mascot->mail.pixmap));
 #else
-    pixbuf = gdk_pixbuf_new_from_file(mascot->mail.pix_file);
+    gdk_pixmap_unref(mascot->mail.pixmap);
 #endif
+  if (mascot->mail.mask)
+#ifdef USE_GTK2
+    g_object_unref(G_OBJECT(mascot->mail.mask));
+#else
+    gdk_pixmap_unref(mascot->mail.mask);
+#endif
+  if(mascot->mail.pix_file){
+  tmp_open=to_utf8(mascot->mail.pix_file);
+#ifdef USE_GTK2
+    pixbuf = gdk_pixbuf_new_from_file(tmp_open, NULL);
+#else
+    pixbuf = gdk_pixbuf_new_from_file(tmp_open);
+#endif
+    g_free(tmp_open);
   }
   else{
     if(mascot->mail.pix_pos==MAIL_PIX_LEFT){
@@ -344,8 +438,13 @@ void LoadBiffPixmap(GtkWidget *widget, typMascot *mascot){
   pixbuf2=gdk_pixbuf_scale_simple(pixbuf,w,h,ipstyle);
   gdk_pixbuf_render_pixmap_and_mask(pixbuf2, &mascot->mail.pixmap,
 				      &mascot->mail.mask, 0xc0);
+#ifdef USE_GTK2
+  g_object_unref(G_OBJECT(pixbuf));
+  g_object_unref(G_OBJECT(pixbuf2));
+#else
   gdk_pixbuf_unref(pixbuf);
   gdk_pixbuf_unref(pixbuf2);
+#endif
   gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->mail.w_draw), w, h);
   gdk_window_resize (widget->window, w, h);
 
@@ -359,7 +458,7 @@ void LoadBiffPixmap(GtkWidget *widget, typMascot *mascot){
 				 mascot->mail.mask,
 				 0, 0 ); 
 }
-
+#endif
 
 // ޥåȤѴݤ˥ХåեƤ
 gint DrawMascot0(typMascot *mascot)
@@ -367,7 +466,11 @@ gint DrawMascot0(typMascot *mascot)
   gint i_work;
 
   for(i_work=0;i_work<2;i_work++){
+#ifdef USE_GTK2
+    gdk_draw_drawable(pixmap_main[i_work],
+#else
     gdk_draw_pixmap(pixmap_main[i_work],
+#endif
 		  mascot->gc_main[i_work],
                   mascot->sprites[mascot->frame_pix[mascot->anime_ptn][mascot->anime_frm]].pixmap,
                   0, 0,
@@ -397,7 +500,11 @@ gint DrawMascot(typMascot *mascot)
        gdk_gc_set_clip_origin(mascot->gc_main[work_page], 0, 0);*/
 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(pixmap_main[work_page],
+#else
   gdk_draw_pixmap(pixmap_main[work_page],
+#endif
 		  mascot->gc_main[work_page],
                   mascot->sprites[mascot->frame_pix[mascot->anime_ptn][mascot->anime_frm]].pixmap,
                   0, 0,
@@ -416,7 +523,11 @@ gint DrawMascot(typMascot *mascot)
 				 0, 0 );
 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(mascot->w_drawing->window,
+#else
   gdk_draw_pixmap(mascot->w_drawing->window,
+#endif
 		  mascot->w_drawing->style->fg_gc[GTK_WIDGET_STATE(mascot->w_drawing)],
 		  pixmap_main[work_page],
 		  0,0,0,0,
@@ -425,7 +536,7 @@ gint DrawMascot(typMascot *mascot)
   mascot->pixmap_page=work_page;
 
 
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
 
 
   gdk_flush();
@@ -441,7 +552,11 @@ gint DrawMascotTemp(typMascot *mascot, gint i_pix)
   //work_page^=1;
 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(pixmap_main[work_page],
+#else
   gdk_draw_pixmap(pixmap_main[work_page],
+#endif
 		  mascot->gc_main[work_page],
                   mascot->sprites[i_pix].pixmap,
                   0, 0,
@@ -456,14 +571,18 @@ gint DrawMascotTemp(typMascot *mascot, gint i_pix)
 				 mascot->sprites[i_pix].mask,
 				 0, 0 ); 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(mascot->w_drawing->window,
+#else
   gdk_draw_pixmap(mascot->w_drawing->window,
+#endif
 		  mascot->w_drawing->style->fg_gc[GTK_WIDGET_STATE(mascot->w_drawing)],
 		  pixmap_main[work_page],
 		  0,0,0,0,
 		  mascot->width, mascot->height);
   mascot->pixmap_page=work_page;
   
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
 
   gdk_flush();
 
@@ -487,8 +606,11 @@ gint DrawMascotWithDigit(typMascot *mascot)
   work_page^=1;
 
 
-
+#ifdef USE_GTK2
+  gdk_draw_drawable(pixmap_main[work_page],
+#else
   gdk_draw_pixmap(pixmap_main[work_page],
+#endif
 		  mascot->gc_main[work_page],
                   mascot->sprites[mascot->frame_pix[mascot->anime_ptn][mascot->anime_frm]].pixmap,
                   0, 0,
@@ -591,14 +713,22 @@ gint DrawMascotWithDigit(typMascot *mascot)
 				 mascot->sprites[mascot->frame_pix[mascot->anime_ptn][mascot->anime_frm]].mask,
 				 0, 0 ); 
 
+#ifdef USE_GTK2
+  gdk_draw_drawable(mascot->w_drawing->window,
+#else
   gdk_draw_pixmap(mascot->w_drawing->window,
+#endif
 		  mascot->w_drawing->style->fg_gc[GTK_WIDGET_STATE(mascot->w_drawing)],
 		  pixmap_main[work_page],
 		  0,0,0,0,
 		  mascot->width, mascot->height);
   mascot->pixmap_page=work_page;
 
-  while (g_main_iteration(FALSE));
+#ifdef USE_GTK2
+  g_object_unref(G_OBJECT(pango_text));
+#endif
+
+  while (my_main_iteration(FALSE));
 
   gdk_flush();
 
@@ -615,17 +745,17 @@ void ReInitGC(typMascot *mascot)
        gdk_gc_unref(mascot->gc_main[i_page]);
        gdk_gc_unref(mascot->gc_mainsd[i_page]);
     }
-    mascot->gc_main[i_page] = CreatePen(pixmap_main[i_page], mascot->colclk);
-    mascot->gc_mainsd[i_page] = CreatePen(pixmap_main[i_page], mascot->colclksd);
+    mascot->gc_main[i_page] = MPCreatePen(pixmap_main[i_page], mascot->colclk);
+    mascot->gc_mainsd[i_page] = MPCreatePen(pixmap_main[i_page], mascot->colclksd);
 
     if(mascot->gc_bal[i_page]){
        gdk_gc_unref(mascot->gc_bal[i_page]);
        gdk_gc_unref(mascot->gc_balbg[i_page]);
        gdk_gc_unref(mascot->gc_balbd[i_page]);
     }
-    mascot->gc_bal[i_page] = CreatePen(pixmap_bal[i_page], mascot->colbal);
-    mascot->gc_balbg[i_page] = CreatePen(pixmap_bal[i_page], mascot->colbalbg);
-    mascot->gc_balbd[i_page] = CreatePen(pixmap_bal[i_page], mascot->colbalbd);
+    mascot->gc_bal[i_page] = MPCreatePen(pixmap_bal[i_page], mascot->colbal);
+    mascot->gc_balbg[i_page] = MPCreatePen(pixmap_bal[i_page], mascot->colbalbg);
+    mascot->gc_balbd[i_page] = MPCreatePen(pixmap_bal[i_page], mascot->colbalbd);
 
     if(mascot->gc_clk[i_page]){
        gdk_gc_unref(mascot->gc_clk[i_page]);
@@ -633,10 +763,10 @@ void ReInitGC(typMascot *mascot)
        gdk_gc_unref(mascot->gc_clkbd[i_page]);
        gdk_gc_unref(mascot->gc_clksd[i_page]);
     }
-    mascot->gc_clk[i_page] = CreatePen(pixmap_clk[i_page], mascot->colclk);
-    mascot->gc_clkbg[i_page] = CreatePen(pixmap_clk[i_page], mascot->colclkbg);
-    mascot->gc_clkbd[i_page] = CreatePen(pixmap_clk[i_page], mascot->colclkbd);
-    mascot->gc_clksd[i_page] = CreatePen(pixmap_clk[i_page], mascot->colclksd);
+    mascot->gc_clk[i_page] = MPCreatePen(pixmap_clk[i_page], mascot->colclk);
+    mascot->gc_clkbg[i_page] = MPCreatePen(pixmap_clk[i_page], mascot->colclkbg);
+    mascot->gc_clkbd[i_page] = MPCreatePen(pixmap_clk[i_page], mascot->colclkbd);
+    mascot->gc_clksd[i_page] = MPCreatePen(pixmap_clk[i_page], mascot->colclksd);
   }
 }
 
@@ -653,7 +783,7 @@ void create_default_font_selection_dialog(typMascot *mascot, gint bal_clk)
   tmp_label=g_malloc0(sizeof(gchar)*256);
 
   fdialog = gtk_dialog_new();
-  gtk_container_border_width(GTK_CONTAINER(fdialog),5);
+  gtk_container_set_border_width(GTK_CONTAINER(fdialog),5);
 
   switch(bal_clk){
   case INIT_DEF_FONT_CLK:
@@ -684,12 +814,14 @@ void create_default_font_selection_dialog(typMascot *mascot, gint bal_clk)
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fdialog)->vbox),
 		     label,TRUE,TRUE,0);
 
+#ifdef __GTK_STOCK_H__
+  button=gtkut_button_new_from_stock(_("OK"),GTK_STOCK_OK);
+#else
   button=gtk_button_new_with_label(_("OK"));
+#endif
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fdialog)->action_area),
 		     button,TRUE,TRUE,0);
-  gtk_signal_connect(GTK_OBJECT(button),"pressed",
-		     GTK_SIGNAL_FUNC(close_child_dialog), 
-		     fdialog);
+  my_signal_connect(button,"clicked",close_child_dialog,fdialog);
 
   gtk_widget_show_all(fdialog);
   gtk_main();
@@ -698,7 +830,7 @@ void create_default_font_selection_dialog(typMascot *mascot, gint bal_clk)
   cdata=g_malloc0(sizeof(confDefFont));
 
   // WinۤϽŤΤExpose٥򤹤٤ƽƤ
-  while (g_main_iteration(FALSE));
+  while (my_main_iteration(FALSE));
 
   
   fdialog = gtk_font_selection_dialog_new(_("Default Font Selection"));
@@ -718,20 +850,14 @@ void create_default_font_selection_dialog(typMascot *mascot, gint bal_clk)
     break;
   }
 
-  gtk_signal_connect(
-       GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fdialog)->ok_button),
-       "clicked", 
-       GTK_SIGNAL_FUNC(ChangeDefFontname), (gpointer)cdata);
+  my_signal_connect(GTK_FONT_SELECTION_DIALOG(fdialog)->ok_button,
+		    "clicked", ChangeDefFontname, (gpointer)cdata);
     
-  gtk_signal_connect(GTK_OBJECT(fdialog),"destroy",
-		     GTK_SIGNAL_FUNC(close_child_dialog), 
-		     fdialog);
+  my_signal_connect(fdialog,"destroy",close_child_dialog, fdialog);
+  
+  my_signal_connect(GTK_FONT_SELECTION_DIALOG(fdialog)->cancel_button,
+		    "clicked", close_child_dialog,fdialog);
   
-  gtk_signal_connect(
-     GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fdialog)->cancel_button),
-     "clicked", 
-     GTK_SIGNAL_FUNC(close_child_dialog),fdialog);
-
   gtk_widget_show(fdialog);
 
   gtk_main();
